#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <sys/wait.h>
#include <time.h>
#include <signal.h>
#include <unistd.h>
#include "alerte.h"
#include "log.h"
#include "s_malloc.h"
#include "loadconfig.h"

char msg[2048];
char cmd_exec[2048];

typedef struct {
	int	pid;
	time_t	time;
} c_pid;

c_pid *pids;
int num_pids;

/*	flag pour suignaler l'effacement (declencche en thread) et bloquer l'ajout
	0: ok, 1: effacement 
*/
int check_pid;

/* initilisation de pids */
void alerte_init(void){
	pids = (c_pid *)MALLOC(sizeof(c_pid));
	num_pids = 0;
	if(pids == NULL){
		snprintf(msg, 2048, "Impossible d'allouer de la memoire a pids\n");
		logger(LOG_ERR, msg);
		exit(-1);
        }
	check_pid=0;
}

/* ajout d'un pid a la liste */
void addpid(int pid){
	c_pid *test;

	/* on incremente le nombre d'entree */
	num_pids++;

	/* reallocation de la memoire */
	test = (c_pid *)REALLOC(pids, num_pids * sizeof(c_pid));
	if(test == NULL){
		snprintf(msg, 2048, "Impossible d'allouer de la memoire a pids\n");
		logger(LOG_ERR, msg);
		FREE(pids);
		exit(1);
	}
	pids = test;

	/* ajout des valeurs */
	pids[num_pids-1].pid = pid;
	pids[num_pids-1].time = time(NULL);
	snprintf(msg, 2048, "pid %i ajoute a la table en position %i au temps : %i", pids[num_pids-1].pid, num_pids-1, (unsigned int)pids[num_pids-1].time);
	logger(LOG_DEBUG, msg);
}

/* supprimer un pid */
void delpid(int pid){
	int i;
	c_pid *b1, *b2, *test;

	/* recopie la table en supprimant l'entree enlevee */
	b1 = pids;
	b2 = pids;
	i = 0;
	num_pids--;
	while(i<num_pids){
		if(b1->pid == pid) b1++;
		b2->pid = b1->pid;
		b2->time = b1->time;
		b1++;
		b2++;
		i++;
	}

	/* si la table n'est pas vide, realloue la memoire */
	if(num_pids>0){
		test = (c_pid *)REALLOC(pids, num_pids * sizeof(c_pid));
		if(test == NULL){
			snprintf(msg, 2048, "Impossible d'allouer de la memoire (%i) a pids\n", num_pids * sizeof(c_pid));
			logger(LOG_ERR, msg);
			FREE(pids);
			exit(1);
		}
		pids = test;
	}
}

void alerte_kill_pid(void){
	int ret;
	int i;
	int status;

	/* si le flag est en place, on quitte la fonction */
	if(check_pid==1){
		logger(LOG_DEBUG, "alerte_kill_pid: check_pid verrouille");
		return;
	}

        /* positionne le flag check */
	logger(LOG_DEBUG, "alerte_kill_pid: verrouillage de check_pid");
        check_pid=1;

	i=0;
	while(i < num_pids){
		snprintf(msg, 2048, "alerte_kill_pid: start waitpid, rec n: %i", i);
		logger(LOG_DEBUG, msg);
		ret = waitpid(pids[i].pid, &status, WNOHANG);
		if(ret > 0)delpid(pids[i].pid);
		i++;
		logger(LOG_DEBUG, "alerte_kill_pid: stop waitpid");
	}

	/* supprime le flag */
	logger(LOG_DEBUG, "alerte_kill_pid: deverrouillage de check_pid");
	check_pid=0;
}

/* verification des pids */
void alerte_check(void){
	int i;
	int status;
	int ret;

	/* si le flag est en place, on quitte la fonction */
	if(check_pid==1){
		logger(LOG_DEBUG, "alerte_check: check_pid verrouille");
		return;
	}

	if(num_pids==0){
		logger(LOG_DEBUG, "aucun pid a traiter");
		return;
	}

	/* positionne le flag check */
	logger(LOG_DEBUG, "alerte_check: verrouillage de check_pid");
	check_pid=1;
	
	i = 0;
	while(i < num_pids){
		/* va voir si le processus fonctionne */
		ret = waitpid(pids[i].pid, &status, WNOHANG);
		snprintf(msg, 2048, "analyse de pid[%i]: %i, temps: %i, code retour = %i", i, pids[i].pid, (unsigned int)pids[i].time, ret);
		logger(LOG_DEBUG, msg);

		/* si il fonctionne mais que son temps est depasse */
		if((time(NULL) - pids[i].time >= config[CF_TIMEOUT].valeur.integer) && (ret==0)){
			snprintf(msg, 2048, "kill de pid[%i]: %i", i, pids[i].pid);
			logger(LOG_ERR, msg);
			
			/* on le tue */
			if(kill(pids[i].pid, 9) < 0){
				snprintf(msg, 2048, "impossible de tuer pid[%i]: %i", i, pids[i].pid);
				logger(LOG_ERR, msg);
			} else {
				waitpid(pids[i].pid, &status, 0);
				delpid(pids[i].pid);
			}
		} else {
			snprintf(msg, 2048, "pid[%i]: %i n'as pas depasse le timeout", i, pids[i].pid);
			logger(LOG_DEBUG, msg);
		}

		/* si il ne fonctionne plus */
		if(ret==-1){
			snprintf(msg, 2048, "pid[%i]: %i est termine, on l'efface de la liste", i, pids[i].pid);
			logger(LOG_DEBUG, msg);
			
			/* on l'efface de la liste */
			delpid(pids[i].pid);
		}
		i++;
	}
	logger(LOG_DEBUG, "Fin du checkup");
	
	/* eneleve le flag check */
	logger(LOG_DEBUG, "alerte_check: deverrouillage de check_pid");
	check_pid=0;
}

/* genere une alerte */
int alerte(char *mac, char *ip){
	int pid;
	int ret;

	/* si le flag est positionne on attend */
	while(check_pid==1){
		logger(LOG_DEBUG, "alerte: check_pid verrouille");
		sleep(1);
	}
	logger(LOG_DEBUG, "alerte: running");

	if(num_pids >= config[CF_MAXTH].valeur.integer){
		logger(LOG_ERR, "Nombre de threads excessifs\n");
		return(-1);
	}

	/* positionne le flag */
	logger(LOG_DEBUG, "alerte: verrouillage de check_pid");
	check_pid=1;

	pid=fork();
	if(pid<0){
		logger(LOG_ERR, "Impossible de forker\n");
		return(pid);
	}
	if(pid>0){
		addpid(pid);
		/* supprime le flag */
		logger(LOG_DEBUG, "alerte: deverrouillage de check_pid");
		check_pid=0;
		return(pid);
	}

	snprintf(msg, 2048, "Demande d'execution de \"%s\"\n", config[CF_ACTION].valeur.string);
	logger(LOG_DEBUG, msg);
	
	ret = execlp(config[CF_ACTION].valeur.string, config[CF_ACTION].valeur.string, mac, ip, NULL);
	if(ret < 0){
		snprintf(msg, 2048, "Erreur lors de l'execution de \"%s\"\n", config[CF_ACTION].valeur.string);
		logger(LOG_ERR, msg);
		exit(1);
	}
	exit(0);
}
