#include <fcntl.h>
#include <signal.h>
#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "loadconfig.h"
#include "log.h"
#include "data.h"
#include "maclist.h"
#include "capture.h"
#include "serveur.h"
#include "alerte.h"
#include "s_malloc.h"

/* intervalle entre deux checkpoint */
#define CHECKPOINT 1

char msg[2048];
void die(int);
void loadconfig(int);
void setsignal(int, void *);
void reloadconfig(int);
void killchild(int);

int main(int argc, char **argv){
	/* Copie des parametre e la lignes de commande pour analyse lors d'un rechargement de la config */
	margc = 0;
	margv = (char **)MALLOC(1);
	while(margc < argc){
		margv = (char **)REALLOC(margv, (margc + 1)*4);
		margv[margc] = (char *)MALLOC(strlen(argv[margc]+1));
		strncpy(margv[margc], argv[margc], strlen(argv[margc])+1);
		margc++;
	}
	
	/* va lire le fichier de configuration */
	config_init();
	config_load();
	
	/* si il le faut, passer le binaire en daemon */
	if(config[CF_DAEMON].valeur.integer == TRUE) daemonize();

	setsignal(SIGINT, (void *)&die);
	setsignal(SIGTERM, (void *)&die);
	setsignal(SIGQUIT, (void *)&die);
	setsignal(SIGABRT, (void *)&die);
	setsignal(SIGHUP, (void *)&reloadconfig);
	setsignal(SIGCHLD, (void *)&killchild);
	setsignal(SIGALRM, (void *)&loadconfig);
	alarm(CHECKPOINT);
	
	/* initialisation de la structure mac */
	data_init();

	/* chargement de la maclist */
	maclist_reload();
	
	/* initilidation du sniffeur */
	cap_init();

	/* initilisation des alertes */
	alerte_init();
	
	/* boucle principale */
	cap_snif();

	/* nettoyage des zones memoires */
	data_close();
	
	/* valeur de retour */
	return(0);
}

void die(int signal){
	snprintf(msg, 2048, "Fini sur le signal: %i\n", signal);
	logger(LOG_DEBUG, msg);
	data_close();
	exit(0);
}

void killchild(int signal){
	alerte_kill_pid();
}

void loadconfig(int signal){
	alerte_check();
	maclist_reload();
	alarm(CHECKPOINT);
}

void reloadconfig(int signal){
	snprintf(msg, 2048, "Signal: %i, rechargement de la configuration\n", signal);
	logger(LOG_DEBUG, msg);
	config_init();
	config_load();
	maclist_reload();
}

void setsignal(int signal, void *function){
	struct sigaction new;
	new.sa_handler = (__sighandler_t)function;
	if (sigaction(signal, &new, NULL)){
		snprintf(msg, 2048, "Impossible de mettre le signal %i en place\n", signal);
		logger(LOG_ERR, msg);
		exit(1);
	}
}
