#include <pcap.h>
#include <stdlib.h>
#include <net/if.h>
#include <net/ethernet.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>
#include "capture.h"
#include "log.h"
#include "loadconfig.h"
#include "data.h"
#include "alerte.h"
#include "s_malloc.h"

#define SNAP_LEN 1514
#define FILTER "arp or rarp"

pcap_t *idcap;
bpf_u_int32 netp, maskp;
struct bpf_program bp;
char msg[2048];

void callback(u_char *, const struct pcap_pkthdr *, const u_char *);

void cap_init(void){
	char err[PCAP_ERRBUF_SIZE];
	char *device;
	
	/* recherche le premier device deisponoible */
	device = NULL;
	if(device == NULL){
		if((device = pcap_lookupdev(err))==NULL){
			snprintf(msg, 2048, "pcap_lookupdev: %s\n", err);
			logger(LOG_ERR, msg);
			exit(-1);
		}
		snprintf(msg, 2048, "device = %s\n", device);
		logger(LOG_DEBUG, msg);
	}

	/* initialise l'interface */
	if((idcap = pcap_open_live(device, SNAP_LEN, 0, 1000, err)) == NULL){
		snprintf(msg, 2048, "pcap_open_live error: %s\n", err);
		logger(LOG_ERR, msg);
		exit(1);
	}

	/* recupere les parametre de l'interface */
	if (pcap_lookupnet(device, &netp, &maskp, err) < 0) {
		snprintf(msg, 2048, "pcap_lookupnet error: %s\n", err);
		logger(LOG_ERR, msg);
		exit(1);
	}

	/* initilise le filtre: */
	if(pcap_compile(idcap, &bp, FILTER, 0x100, maskp) < 0){
		snprintf(msg, 2048, "pcap_compile error: %s\n", pcap_geterr(idcap));
		logger(LOG_ERR, msg);
		exit(1);
	}

	/* appliquer le filtre: */
	if(pcap_setfilter(idcap, &bp)<0){
		snprintf(msg, 2048, "pcap_setfilter error: %s\n", pcap_geterr(idcap));
		logger(LOG_ERR, msg);
		exit(1);
	}
	snprintf(msg, 2048, "pcap_setfilter [%s]: ok\n", FILTER);
	logger(LOG_DEBUG, msg);
}


void cap_snif(void){
	if(pcap_loop(idcap, 0, callback, NULL) <0){
		snprintf(msg, 2048, "pcap_loop error: %s\n", pcap_geterr(idcap));
		logger(LOG_ERR, msg);
		exit(1);
	}
}

void callback(u_char *user, const struct pcap_pkthdr *h, const u_char *buff){
	data_mac macs;
	char smacs[18];
	char ip[16];
	struct ether_header *eh;
	struct ether_arp *ea;
	int i;
	int ret;

	logger(LOG_DEBUG, "Taritement Capture");

	eh = (struct ether_header *)buff;
	ea = (struct ether_arp *)(eh + 1);
	i=-1;
	while(i++<5) macs.octet[i].octet = ea->arp_sha[i];
	
	data_tomac(macs, smacs);
	snprintf(ip, 16, "%u.%u.%u.%u", ea->arp_spa[0], ea->arp_spa[1], ea->arp_spa[2], ea->arp_spa[3]);

	i = data_exist(&macs);
	if(i==NOT_EXIST){
		snprintf(msg, 2048, "Mac sender: %s, %s : ERREUR\n", smacs, ip);
		logger(LOG_ERR, msg);
		ret = alerte(smacs, ip);
		if(ret > 0){
			snprintf(msg, 2048, "Forked as pid: %i\n", ret);
			logger(LOG_DEBUG, msg);	
		}
	} else {
		snprintf(msg, 2048, "Mac sender: %s : OK\n", smacs);
		logger(LOG_DEBUG, msg);
	}

	logger(LOG_DEBUG, "Taritement Capture: Fin");
}
