#include <stdio.h>
#include <malloc.h>
#include "data.h"
#include "s_malloc.h"
#include "log.h"

/* Taille de la table de hachage (nombre premier) */
#define HASH_SIZE 1999
#define HASH_B 256

/* conversion bin -> hexa */
const char conv[16] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};

/* conversion hexa -> bin */
const unsigned char vnoc[103] = {
	0,0,0,0,0,0,0,0,0,0,            /*  9 */
	0,0,0,0,0,0,0,0,0,0,            /* 19 */
	0,0,0,0,0,0,0,0,0,0,            /* 29 */
	0,0,0,0,0,0,0,0,0,0,            /* 39 */
	0,0,0,0,0,0,0,0,0,1,            /* 49 */
	2,3,4,5,6,7,8,9,0,0,            /* 59 */
	0,0,0,0,0,10,11,12,13,14,       /* 69 */
	15,0,0,0,0,0,0,0,0,0,           /* 79 */
	0,0,0,0,0,0,0,0,0,0,            /* 89 */
	0,0,0,0,0,0,0,10,11,12,         /* 99 */
	13,14,15                        /* 102 */
};

/* Pour les logs: */
char msg[4096];

unsigned int data_hash(data_mac *mac);

/* unite de la table */
typedef struct __data_element {
	data_mac mac;
	struct __data_element *next;
} data_element;

/* pointeur sur unit� */
typedef data_element *p_data_element;

/* Tableau d'adresses mac */
p_data_element *data_tab;

/* initialise la memoire */
void data_init(void){
	p_data_element *init_null;

	/* reserve le tableau de pointeurs de la taille du hash */
	data_tab = (p_data_element *)MALLOC(HASH_SIZE * sizeof(p_data_element));

	/* met tous ses element a NULL */
	init_null = data_tab;
	while(init_null < data_tab + HASH_SIZE){
		*init_null = NULL;
		init_null++;
	}
	
}

/* libere la memoire */
void data_reset(void){
	p_data_element *init_null;
	data_element *del, *delnext;
	init_null = data_tab;
	while(init_null < data_tab + HASH_SIZE){
		del = *init_null;
		while(1){
			if(del != NULL){
				delnext = (*del).next;
				FREE(del);
				del = delnext;
			} else {
				break;
			}
		}
		*init_null = NULL;
		init_null++;
	}
}

/* libere la memoire */
void data_close(void){
	data_reset();
	FREE(data_tab);
}

/* ajoute une @ mac a la table */
void data_add(data_mac *mac){
	p_data_element *add;
	data_element *libre;
	add = data_tab;

	/* possitionement de la nouvelle donn� en memoire */
	add += data_hash(mac);

	/* recherche d'un emplacement libre */
	if(*add != NULL){
		libre = *add;
		while((*libre).next!= NULL){
			libre = (*libre).next;
		}
		add = &((*libre).next);
	}

	/* creation et malloc d'un nouveau data_element (nouvelle mac) dans add */
	libre = (data_element *)MALLOC(sizeof(data_element));

	if(libre ==NULL){
		snprintf(msg, 4096, "%s %i Erreur d'allocation memoire\n", __FILE__, __LINE__);
		logger(LOG_ERR, msg);
	} else {
		(*libre).mac.copie.a = (*mac).copie.a;
		(*libre).mac.copie.b = (*mac).copie.b;
		(*libre).next = NULL;
		*add = libre;
		#ifdef DEBUG
			char buf[18];
			data_tomac(*mac, buf);
			snprintf(msg, 4096, "%s %i Adresse ajoutee en memoire: %s\n", __FILE__, __LINE__, buf);
			logger(LOG_DEBUG, msg);
		#endif
	}
}

/* cherche si @ mac existe */
int data_exist(data_mac *mac){
	data_element *question;
	p_data_element *emplacement;

	/* va chercher le segement dans la table de hachage */
	emplacement = data_tab;
	emplacement += data_hash(mac);
	#ifdef DEBUG
		snprintf(msg, 4096, "%s %i Adresse de l'emplacement teste: 0x%x\n", __FILE__, __LINE__, emplacement);
		logger(LOG_DEBUG, msg);
	#endif
	question = *emplacement;

	if(question == NULL)return(NOT_EXIST);

	while(((*question).mac.copie.a != (*mac).copie.a) || ((*question).mac.copie.b != (*mac).copie.b)){
		if((*question).next != NULL){
			question = (*question).next;
		} else {
			return(NOT_EXIST);
		}
	}
	return(EXIST);
}

/* fait le hachage */
unsigned int data_hash(data_mac *mac){
	unsigned int v = 0;
	unsigned int i = 0;

	while(i<6){
		v = (v * HASH_B + mac[0].octet[i].octet) % HASH_SIZE;
		i++;
	}
	return(v);
}

/* conversion du type mac vers un string lisible */
void data_tomac(data_mac bin, char *buf){
	buf[0]=conv[bin.octet[0].hex.hi];
	buf[1]=conv[bin.octet[0].hex.low];
	buf[2]=':';
	buf[3]=conv[bin.octet[1].hex.hi];
	buf[4]=conv[bin.octet[1].hex.low];
	buf[5]=':';
	buf[6]=conv[bin.octet[2].hex.hi];
	buf[7]=conv[bin.octet[2].hex.low];
	buf[8]=':';
	buf[9]=conv[bin.octet[3].hex.hi];
	buf[10]=conv[bin.octet[3].hex.low];
	buf[11]=':';
	buf[12]=conv[bin.octet[4].hex.hi];
	buf[13]=conv[bin.octet[4].hex.low];
	buf[14]=':';
	buf[15]=conv[bin.octet[5].hex.hi];
	buf[16]=conv[bin.octet[5].hex.low];
	buf[17]=0;
}

/* conversion d'un string lisible (table arp du noyaux, ...) vers un type mac */
void data_tohex(unsigned char *macaddr, data_mac *voila){
	data_octet toto;
	toto.hex.hi = vnoc[macaddr[0]];
	toto.hex.low = vnoc[macaddr[1]];
	voila->octet[0] = toto;
	toto.hex.hi = vnoc[macaddr[3]];
	toto.hex.low = vnoc[macaddr[4]];
	voila->octet[1] = toto;
	toto.hex.hi = vnoc[macaddr[6]];
	toto.hex.low = vnoc[macaddr[7]];
	voila->octet[2] = toto;
	toto.hex.hi = vnoc[macaddr[9]];
	toto.hex.low = vnoc[macaddr[10]];
	voila->octet[3] = toto;
	toto.hex.hi = vnoc[macaddr[12]];
	toto.hex.low = vnoc[macaddr[13]];
	voila->octet[4] = toto;
	toto.hex.hi = vnoc[macaddr[15]];
	toto.hex.low = vnoc[macaddr[16]];
	voila->octet[5] = toto;
}
