#include <stdio.h>
#include <malloc.h>
#include <getopt.h>
#include <string.h>
#include <stdlib.h>
#include "loadconfig.h"
#include "log.h"
#include "s_malloc.h"

char msg[4096];

void miseenforme(char*);
void miseenmemoire(char*);
char lowercase(char);
int convert_int(char*);
int convert_boolean(char*);

#define NUM_PARAMS 9

void config_init(void){
	char c;
	extern char *optarg;
	extern int optind;
	extern int opterr;
	
 	optind = 0;
	if(config==NULL)config = (config_cell *)MALLOC(sizeof(config_cell) * NUM_PARAMS);

	/* chargement des valeurs par defaut et definition des entrées */
	config[0].type = 0;
	strncpy(config[0].attrib, "maclist file", 512);
	strncpy(config[0].valeur.string, "/etc/mac/maclist", 1024);
	
	config[1].type = 0;
	strncpy(config[1].attrib, "log file", 512);
	strncpy(config[1].valeur.string, "/var/log/mac.log", 1024);
	
	config[2].type = 0;
	strncpy(config[2].attrib, "action on detect", 512);
	strncpy(config[2].valeur.string, "/sbin/scripts/arpalert.pl %s >>/tmp/ou.log", 1024);
	
	config[3].type = 0;
	strncpy(config[3].attrib, "lock file", 512);
	strncpy(config[3].valeur.string, "/var/run/mac.lock", 1024);
	
	config[4].type = 2;
	strncpy(config[4].attrib, "daemon", 512);
	config[4].valeur.integer = TRUE;
	
	config[5].type = 1;
	strncpy(config[5].attrib, "reload interval", 512);
	config[5].valeur.integer = 600;
	
	config[6].type = 1;
	strncpy(config[6].attrib, "log level", 512);
	config[6].valeur.integer = 6;
	
	config[7].type = 1;
	strncpy(config[7].attrib, "execution timeout", 512);
	config[7].valeur.integer = 10;

	config[8].type = 1;
	strncpy(config[8].attrib, "max threads", 512);
	config[8].valeur.integer = 20;

	/* cherge / recharge les parametres de la ligne de commande */
	strncpy(config_file, CONFIG_FILE, 2048);
	while ((c = getopt(margc, margv, "f:")) != EOF) {
		switch (c) {
			case 'f': 
				strncpy(config_file, optarg, 2048);
			break;
			case '?':
				snprintf(msg, 4096, "%s %i option inconue\n", __FILE__, __LINE__);
				fprintf(stderr, msg);
				exit(1);
			break;
		}
	}

}

void config_load(void){
	FILE *fp;
	char *buf;

	buf = (char *)malloc(4096);

	if((fp=fopen(config_file, "r"))==NULL){
		snprintf(msg, 4096, "%s %i fopen: %s - ouverture impossible\n", __FILE__, __LINE__, config_file);
		fprintf(stderr, msg);
		exit(1);
	}	

	while((buf = fgets(buf, 4096, fp)) != NULL){
		miseenforme(buf);
		if(*buf!=0){
			miseenmemoire(buf);
		}
	}
	
	free(buf);
	fclose(fp);
}

void miseenmemoire(char *buf){
	char *src;
	char *m_eq;
	char *m_end;
	char *gauche, *g;
	char *droite, *d;	
	int i;
	int protection;
	
	gauche = (char *)malloc(4096);
	droite = (char *)malloc(4096);
	g = gauche;
	d = droite;
	src = buf;	

	protection = 0;
	while(*src != 0){
		if(*src == '"'){
			protection ^= 0xff;
			src++;
			continue;
		}
		if(protection != 255 && *src=='=')m_eq=src;
		src++;
	}
	m_end=src;
	if(*m_eq!='='){
		fprintf(stderr, "%i: erreur dans le fichier de config a la ligne: %s\n", __LINE__, buf);
		exit(1);
	}
	if(*(m_eq-1)!=' '){
		fprintf(stderr, "%i: erreur dans le fichier de config a la ligne: %s\n", __LINE__, buf);
		exit(1);
	}
	if(*(m_eq-1)!=' '){
		fprintf(stderr, "%i: erreur dans le fichier de config a la ligne: %s\n", __LINE__, buf);
		exit(1);
	}
	src=buf;
	while(src<(m_eq-1)){
		*gauche=lowercase(*src);
		gauche++;
		src++;
	}
	*gauche=0;
	src+=3;
	if(*src=='"'){
		src++;
		m_end--;
	}
	while(src<m_end){
		*droite=*src;
		droite++;
		src++;
	}
	*droite=0;

	i = 0;
	while(i < NUM_PARAMS){
		if(strncmp(g, config[i].attrib, 512)==0){
			switch(config[i].type){
				case 0: strncpy(config[i].valeur.string, d, 1024); break;
				case 1: config[i].valeur.integer = convert_int(d); break;
				case 2: config[i].valeur.integer = convert_boolean(d); break;
			}
		}
		i++;
	}
}

int convert_int(char *buf){
	int res = 0;
	int count = 1;
	while(*buf != 0){
		res *= 10;
		res += *buf-48;
		count *= 10;
		buf++;
	}	
	return res;
}

int convert_boolean(char *buf){
	char *src;
	src = buf;
	*src = lowercase(*src);
	switch(*src){
		case 'o': return(TRUE);
		case 't': return(TRUE);
		case 'y': return(TRUE);

		case 'n': return(FALSE);
		case 'f': return(FALSE);
	}
	return(FALSE);
}

char lowercase(char in){
	if(in > 64 && in < 91)in+=32;
	return in;
}

void miseenforme(char *params){
	char *src;
	char *dst;
	char *fin;
	char *mem;
	int protection;
	int debut;
	int space;

        /* suppression des commanteaires */
	src = params;
	protection = 0;
	while(*src != 0){
		if(*src == '"'){
			protection ^= 0xff;
			src++;
			continue;
		}
		if(protection != 255 && *src == '#'){
			*src=0;
			break;
		}
		src++;
	}

	/* suppression des espaces inutiles */
	debut = 0;
	space = 0;	
	src = params;
	dst = params;
	protection = 0;
	while(*src != 0){
		/* on conserve l'interieur des guillemets sans modifs */
		if(*src == '"' || protection == 255){
			*dst = *src;
			if(*src == '"'){
				protection ^= 0xff; /* ou  exclusif, inversion de octets pour le flag*/
			}
			src++;
			dst++;
			continue;
		}

		/* on ignore les saut de lignes */
		if(*src == '\n' || *src == '\r'){
			src++;
			continue;
		}

		/* si on a un caracteres non blanc on le recopie */
		if(!(*src == ' ' || *src == '\t')){
			debut = 1;
			*dst = *src;
			dst++;
			src++;
			space = 0;
			continue;
		}

		/* dans les autres cas, on considere que c'est un espace, si ce n'est pas le premier, on met un espace */
		if(space == 0 && debut == 1){
			*dst=' ';
			dst++;
		}
		space = 1;
		src++;
	}
	*dst=0;
	fin = dst;	

	/* suppresion de l'eventuel dernier espace */
	if(*(fin-1)==' '){
		fin--;
		dst--;
		*fin=0;
	}

	/* mise en forme des egales coll�: peuvent etre mot=mot */
	src = params;
	protection = 0;
	while(*src != 0){
		if(*src == '"'){
			protection ^= 0xff;
			src++;
			continue;
		}
		if(protection != 255 && *src == '=')break;
		src++;
	}
	if(*src == '='){
		mem = src;
		if(*(src+1)!=' ')fin++;
		if(*(src-1)!=' '){fin++; src++;}

		*(fin+1)=0;
		while(fin!=mem){
			*fin=*dst;
			fin--;
			dst--;
		}

		*src = '=';
		*(src-1) = ' ';
		*(src+1) = ' ';
	}
}
